# Copyright 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Add order column

Revision ID: 28ac1ba9c3db
Revises: 13d819bbb0ff
Create Date: 2014-08-02 21:54:55.634109

"""

# revision identifiers, used by Alembic.
revision = '28ac1ba9c3db'
down_revision = '13d819bbb0ff'

from alembic import op
import sqlalchemy as sa


def upgrade():
    op.add_column('runs', sa.Column('run_at', sa.DateTime()))


def downgrade():
    op.drop_column('runs', 'run_order')
